/* Copyright (c) 2011 Panev Maxim (panevmv@gmail.com)
 * Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php)
 * and GPL (http://www.opensource.org/licenses/gpl-license.php) licenses.
 *
 * $LastChangedDate: 2011-06-23 15:24:00 +0300 $
 *
 * Requires: 	jQuery 1.1.3+
 * 				jQuery Dimensions 1.1.2+
 */

jQuery.fn.keyboard = function(options){
	var keyboards = [];
	var options = jQuery.extend({
		type: 'full',
		language: 'russian', /* russian|english */
		capslock: false,
		state: 'literals', /* literals|digits */
		field: null, /* поле, в которое нужно вставлять данные с клавиатуры */
		hidden_buttons: [], /* клавиши, которые требуется скрыть */
		padding: [false, false, false, false, false], /* пользовательские отступы каждой строки. FALSE - отступы по умолчанию */
		captions: { // подписи на кнопках
			enter: '' // пока можно менять только надпись на Enter'е, в других нет необходимости
		},
		autoLower: false, // признак, что в случае верхнего регистра после первой введенной большой буквы нужно сразу же переключиться в маленькие
		onSetValue: null, /* пользовательская функция обработки установки значения поля ввода */
		onEnter: null /* пользовательская процедура обработки нажатия ввода */
	},options);
	var setValue = function(value){
		if ($(options.field).is('input')) {
			$(options.field).val(value);
		} else {
			$(options.field).attr('value',value);
			$(options.field).html(value);
		}
		if (options.onSetValue!=null) options.onSetValue(value);
	};
	var getValue = function(){
		if ($(options.field).is('input')) {
			var res = $(options.field).val();
		} else {
			var res = $(options.field).html();
		}
		return res;
	};
	var keys = [
		{
			condition: "((options.type=='full')&&(options.language=='russian')&&(options.capslock==false)&&(options.state=='literals'))",
			lines: [
				{
					padding: '0px 0px 0px 2em',
					items: ['1','2','3','4','5','6','7','8','9','0','backspace']//12
				},
				{
					padding: '0px',
					items: ['й','ц','у','к','е','н','г','ш','щ','з','х']//12
				},
				{
					padding: '0px 0px 0px 2em',
					items: ['ф','ы','в','а','п','р','о','л','д','ж','э']//11
				},
				{
					padding: '0px 0px 0px 4em',
					items: ['я','ч','с','м','и','т','ь','б','ю','ъ','digits']//11
				},
				{
					padding: '0px',
					items: ['capslock','language','space','clear','enter']
				}
			]
		},
		{
			condition: "((options.type=='full')&&(options.language=='russian')&&(options.capslock==true)&&(options.state=='literals'))",
			lines: [
				{
					padding: '0px 0px 0px 2em',
					items: ['1','2','3','4','5','6','7','8','9','0','backspace']//12
				},
				{
					padding: '0px',
					items: ['Й','Ц','У','К','Е','Н','Г','Ш','Щ','З','Х']//12
				},
				{
					padding: '0px 0px 0px 2em',
					items: ['Ф','Ы','В','А','П','Р','О','Л','Д','Ж','Э']//11
				},
				{
					padding: '0px 0px 0px 4em',
					items: ['Я','Ч','С','М','И','Т','Ь','Б','Ю','Ъ','digits']//11
				},
				{
					padding: '0px',
					items: ['capslock','language','space','clear','enter']
				}
			]
		},
		{
			condition: "((options.type=='full')&&(options.language=='english')&&(options.capslock==false)&&(options.state=='literals'))",
			lines: [
				{
					padding: '0px 0px 0px 2em',
					items: ['1','2','3','4','5','6','7','8','9','0','backspace']//12
				},
				{
					padding: '0px',
					items: ['q','w','e','r','t','y','u','i','o','p','-']//12
				},
				{
					padding: '0px 0px 0px 2em',
					items: ['a','s','d','f','g','h','j','k','l',';','\'']//11
				},
				{
					padding: '0px 0px 0px 4em',
					items: ['z','x','c','v','b','n','m',',','.','/','digits']//11
				},
				{
					padding: '0px',
					items: ['capslock','language','space','clear','enter']
				}
			]
		},
		{
			condition: "((options.type=='full')&&(options.language=='english')&&(options.capslock==true)&&(options.state=='literals'))",
			lines: [
				{
					padding: '0px 0px 0px 2em',
					items: ['1','2','3','4','5','6','7','8','9','0','backspace']//12
				},
				{
					padding: '0px',
					items: ['Q','W','E','R','T','Y','U','I','O','P','_']//12
				},
				{
					padding: '0px 0px 0px 2em',
					items: ['A','S','D','F','G','H','J','K','L',':','"']//11
				},
				{
					padding: '0px 0px 0px 4em',
					items: ['Z','X','C','V','B','N','M','@','!','?','digits']//11
				},
				{
					padding: '0px',
					items: ['capslock','language','space','clear','enter']
				}
			]
		}
	];
	
	var redraw = function(target) {
		var dom = $('<div class="ui-keyboard keyboard-type-'+options.type+'"></div>');
		var index = -1;
		// сначала определяем, какой из элементов нам подходит для текущего представления
		$.each(keys,function(i,e){
			evaluate = 'res='+e.condition+';';
			eval(evaluate);
			if (res) {
				index=i;
				return false;
			}
		});
		if (index==-1) return false;
		var item_height = jQuery(target).height()/keys[index].lines.length/1.2;
		var item_width = jQuery(target).width()/13.2;
		item_width = item_height*1.2;
		
		for (var line=0; line<keys[index].lines.length; line++) {
			var dom_line = $('<div class="ui-keyboard-line"></div>')
				.css({
					width: '100%',
					height: item_height*1.2+'px',
					padding: options.padding[line] ? options.padding[line] : keys[index].lines[line].padding
				});
			for (var key=0; key<keys[index].lines[line].items.length; key++) {
				switch (keys[index].lines[line].items[key]) {
				case 'backspace':
					if ($.inArray('backspace',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key backspace"><span>&larr;</span></div>')
						.css({width: item_width});
						$("span",dom_key)
						.css({
							'line-height': item_height+'px',
							'font-size': item_height/1.5+'px'
						});
					}
					break;
				case 'capslock':
					if ($.inArray('capslock',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key capslock"><span>'+(options.capslock?'в нижнем<br />регистре':'В ВЕРХНЕМ<br />РЕГИСТРЕ')+'</span></div>')
						.css({width: item_width*2});
						$("span",dom_key)
						.css({
							'line-height': item_height/2+'px',
							'font-size': item_height/3+'px'
						});
					}
					break;
				case 'digits':
					if ($.inArray('digits',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key digits"><span>Цифры и<br />символы</span></div>')
						.css({width: item_width*1.3});
						$("span",dom_key)
						.css({
							'line-height': item_height/2+'px',
							'font-size': item_height/3+'px'
						});
					}
					break;
				case 'language':
					if ($.inArray('language',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key language">'+(options.language=='english'?'<div class="russian"><img src="./images/kbrd_icon-rus_90.png" /><span>Русский</span></div>':'<div class="english"><img src="./images/kbrd_icon-eng_90.png" /><span>English</span></div>')+'</div>')
						.css({
							width: item_width*1.3
						});
					}
					break;
				case 'space':
					if ($.inArray('space',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key space"><span class="value"> </span></div>')
						.css({width: item_width*(options.enter?5:6)});
					}
					break;
				case 'clear':
					if ($.inArray('clear',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key clear"><span>Стереть всё</span></div>')
						.css({width: item_width*2});
						$("span",dom_key)
						.css({
							'line-height': item_height+'px',
							'font-size': item_height/3+'px'
						});
					}
					break;
				case 'enter':
					if ($.inArray('enter',options.hidden_buttons)==-1) {
						var dom_key = $('<div class="ui-keyboard-key enter"><span>'+(options.captions.enter==''?'Ввод':options.captions.enter)+'</span></div>')
						.css({width: item_width*2});
						$("span",dom_key)
						.css({
							'line-height': item_height+'px',
							'font-size': item_height/2+'px'
						});
					}
					break;
				default:
					var dom_key = $('<div class="ui-keyboard-key key"><span class="value">'+keys[index].lines[line].items[key]+'</span></div>')
					.css({width: item_width});
					$("span",dom_key)
					.css({
						'line-height': item_height+'px',
						'font-size': item_height/2+'px'
					});
					break;
				}
				dom_key.css({height: item_height}).appendTo(dom_line);
			}
			dom_line.appendTo(dom);
		}
		jQuery(target).html(dom);

		jQuery('.key',target)
		.unbind('click')
		.click(function(){
			var current = getValue();
			var add = $('.value',this).html();
			setValue(current+add);
			if (options.autoLower) {
				options.capslock = false;
				redraw(target);
			}
		});
		jQuery('.clear',target)
		.unbind('click')
		.click(function(){
			setValue('');
		});
		jQuery('.space',target)
		.unbind('click')
		.click(function(){
			setValue(getValue()+' ');
		});
		jQuery('.backspace',target)
		.unbind('click')
		.click(function(){
			buff = getValue();
			setValue(buff.substr(0,buff.length-1));
		});
		jQuery('.capslock',target)
		.unbind('click')
		.click(function(){
			options.capslock = !options.capslock;
			redraw(target);
		});
		jQuery('.language',target)
		.unbind('click')
		.click(function(){
			options.language = options.language=='russian' ? 'english': 'russian';
			redraw(target);
		});
		jQuery('.enter',target)
		.unbind('click')
		.click(function(){
			if (options.onEnter!=null) options.onEnter(getValue());
		});
	};
	
	return this.each(function() {
		redraw(this);
		keyboards.push(this);
	});
	
};
